#include "Prefab.h"
#include "../utils/Logging.h"
#include "../utils/utils.h"
#include "../../Entitys/Object.h"
#include "../../core/utils/AssetManager.h"

#include <yaml-cpp/yaml.h>
#include <fstream>
#include <filesystem>
#include <cctype>
#include <ctime>
#include <sstream>

std::string Prefab::CreatePrefab(const std::shared_ptr<Object>& obj, const std::string& outputDir)
{
    if (!obj)
    {
        Logger::LogError("[Prefab] Cannot create prefab from null object.");
        return "";
    }

    std::string name = obj->GetName().empty() ? "Unnamed" : obj->GetName();
    for (char& c : name)
    {
        if (!std::isalnum(c) && c != '_' && c != '-' && c != ' ')
            c = '_';
    }

    // Emit object YAML
    YAML::Emitter objectOut;
    objectOut.SetIndent(2);
    obj->Save(objectOut);

    // Wrap metadata
    YAML::Emitter out;
    out.SetIndent(2);

    std::string hash = HashString(objectOut.c_str());
    std::time_t now = std::time(nullptr);

    out << YAML::BeginMap;
    out << YAML::Key << "uaid"         << YAML::Value << AssetManager::GenerateUAID();
    out << YAML::Key << "filename"     << YAML::Value << name + ".cpfb";
    out << YAML::Key << "filetype"     << YAML::Value << "cpfb";
    out << YAML::Key << "type"         << YAML::Value << "Prefab";
    out << YAML::Key << "prefabName"   << YAML::Value << name;
    out << YAML::Key << "hash"         << YAML::Value << hash;
    out << YAML::Key << "lastModified" << YAML::Value << now;
    out << YAML::Key << "loaded"       << YAML::Value << true;
    out << YAML::Key << "prefabYAML"   << YAML::Value << objectOut.c_str();
    out << YAML::EndMap;

    // Output file
    std::filesystem::create_directories(outputDir);
    std::string fullPath = outputDir + "/" + name + ".cpfb";

    std::ofstream file(fullPath);
    if (file.is_open())
    {
        file << out.c_str();
        file.close();
        Logger::LogInfo("[Prefab] Saved prefab: %s", fullPath.c_str());
        return fullPath;
    }
    else
    {
        Logger::LogError("[Prefab] Failed to save prefab: %s (%s)", fullPath.c_str(), std::strerror(errno));
        return "";
    }
}
